package com.example.groracle.Brokers;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.dussan.vaadin.dcharts.DCharts;
import org.dussan.vaadin.dcharts.base.elements.XYaxis;
import org.dussan.vaadin.dcharts.base.elements.XYseries;
import org.dussan.vaadin.dcharts.data.DataSeries;
import org.dussan.vaadin.dcharts.data.Ticks;
import org.dussan.vaadin.dcharts.metadata.LegendPlacements;
import org.dussan.vaadin.dcharts.metadata.SeriesToggles;
import org.dussan.vaadin.dcharts.metadata.renderers.AxisRenderers;
import org.dussan.vaadin.dcharts.metadata.renderers.LegendRenderers;
import org.dussan.vaadin.dcharts.metadata.renderers.SeriesRenderers;
import org.dussan.vaadin.dcharts.options.Axes;
import org.dussan.vaadin.dcharts.options.Highlighter;
import org.dussan.vaadin.dcharts.options.Legend;
import org.dussan.vaadin.dcharts.options.Options;
import org.dussan.vaadin.dcharts.options.Series;
import org.dussan.vaadin.dcharts.options.SeriesDefaults;
import org.dussan.vaadin.dcharts.renderers.legend.EnhancedLegendRenderer;

import com.example.groracle.DAL.DAO.StatLinePerDateDAO;
import com.example.groracle.DAL.DAO.StatLinePerHostDAO;
import com.example.groracle.DAL.DAO.StatLinePerPatternDAO;
import com.example.groracle.Entity.Agregated.StatLinePerDate;
import com.example.groracle.Entity.Agregated.StatLinePerHost;
import com.example.groracle.Entity.Agregated.StatLinePerPattern;
/**
 * Class handling generating statistics
 * @author tdosek
 * @version 1.0
 */
public class Statistics{
    private String type = null;
    /**
     * Constructor of the class
     * @param type Type of the statistics we want to generate
     */
    public Statistics(String type){
	this.type = type;
    }
    /**
     * Method which draws the statistics and returns them in a form of DCharts widget
     * @return statistics in form of DCharts widget
     */
    public DCharts drawStatistics(){
	DataSeries data = new DataSeries();
	Ticks descriptions = new Ticks();
	Axes axes = new Axes();
	Series series = new Series();
	switch (type){
	case "LinesPerPattern": {
	    // Prepare chart objects
	    SeriesDefaults seriesDefaults = new SeriesDefaults()
	        .setRenderer(SeriesRenderers.BAR);
	    Highlighter highlighter = new Highlighter()
	        .setShow(false);
	    descriptions.add("Pattern");
	    // Fill in actual data
	    StatLinePerPatternDAO sDao = new StatLinePerPatternDAO();
	    List<StatLinePerPattern> results = sDao.getStat();
	    if (results.size()>0){
		for (StatLinePerPattern i: results){
		    data.add(i.getCount());
		    series.addSeries(new XYseries().setLabel(i.getPattern()));
	    	}
	    }
	    sDao.closeSession();
	    axes.addAxis(new XYaxis().setRenderer(AxisRenderers.CATEGORY).setTicks(descriptions));
	    Legend legend = new Legend()
		.setShow(true)
		.setRenderer(LegendRenderers.ENHANCED)
		.setRendererOptions(
			new EnhancedLegendRenderer()
				.setSeriesToggle(SeriesToggles.SLOW)
				.setSeriesToggleReplot(true))
		.setPlacement(LegendPlacements.OUTSIDE_GRID);
	    Options options = new Options()
	        .setSeriesDefaults(seriesDefaults)
	        .setSeries(series)
	        .setAxes(axes)
	        .setLegend(legend)
	        .setHighlighter(highlighter);
	    DCharts chart = new DCharts()
	        .setDataSeries(data)
	        .setOptions(options)
	        .show();
	    return chart;
	}
	case "LinesPerHost": {
	    // Prepare chart objects
	    SeriesDefaults seriesDefaults = new SeriesDefaults()
	        .setRenderer(SeriesRenderers.BAR);
	    Highlighter highlighter = new Highlighter()
	        .setShow(false);
	    descriptions.add("Host");
	    // Fill in actual data
	    StatLinePerHostDAO sDao = new StatLinePerHostDAO();
	    List<StatLinePerHost> results = sDao.getStat();
	    if (results.size()>0){
		for (StatLinePerHost i: results){
		    data.add(i.getCount());
		    series.addSeries(new XYseries().setLabel(i.getHost()));
	    	}
	    }
	    sDao.closeSession();
	    axes.addAxis(new XYaxis().setRenderer(AxisRenderers.CATEGORY).setTicks(descriptions));
	    Legend legend = new Legend()
		.setShow(true)
		.setRenderer(LegendRenderers.ENHANCED)
		.setRendererOptions(
			new EnhancedLegendRenderer()
				.setSeriesToggle(SeriesToggles.SLOW)
				.setSeriesToggleReplot(true))
		.setPlacement(LegendPlacements.OUTSIDE_GRID);
	    Options options = new Options()
	        .setSeriesDefaults(seriesDefaults)
	        .setSeries(series)
	        .setAxes(axes)
	        .setLegend(legend)
	        .setHighlighter(highlighter);
	    DCharts chart = new DCharts()
	        .setDataSeries(data)
	        .setOptions(options)
	        .show();
	    return chart;
	}
	case "LoadInTime": {
	    // Prepare chart objects	    
	    SeriesDefaults seriesDefaults = new SeriesDefaults()
	        .setRenderer(SeriesRenderers.BAR);
	    Highlighter highlighter = new Highlighter()
	        .setShow(false);
	    descriptions.add("Date");
	    // Fill in actual data
	    StatLinePerDateDAO sDao = new StatLinePerDateDAO();
	    List<StatLinePerDate> results = sDao.getStat();
	    if (results.size()>0){
		for (StatLinePerDate i: results){
		   data.add(i.getCount());
		   series.addSeries(new XYseries().setLabel(i.getDate()));   
	    	}
	    }
	    sDao.closeSession();
	    axes.addAxis(new XYaxis().setRenderer(AxisRenderers.CATEGORY).setTicks(descriptions));
	    Legend legend = new Legend()
		.setShow(true)
		.setRenderer(LegendRenderers.ENHANCED)
		.setRendererOptions(
			new EnhancedLegendRenderer()
				.setSeriesToggle(SeriesToggles.SLOW)
				.setSeriesToggleReplot(true))
		.setPlacement(LegendPlacements.OUTSIDE_GRID);
	    Options options = new Options()
	        .setSeriesDefaults(seriesDefaults)
	        .setSeries(series)
	        .setAxes(axes)
	        .setLegend(legend)
	        .setHighlighter(highlighter);
	    DCharts chart = new DCharts()
	        .setDataSeries(data)
	        .setOptions(options)
	        .show();
	    return chart;
	}
	}
	return null;
    }

}
