package com.example.groracle.DAL.DAO;
import java.util.List;

import org.hibernate.SQLQuery;
import org.hibernate.Session;

import com.example.groracle.DAL.SessionFactoryUtil;
import com.example.groracle.Entity.Group;
/**
 * Class implementing operations on top of Group object
 * @author tdosek
 * @version 1.1
 */
public class GroupDAO{
    private Session ss = null;
    /**
     * Constructor of the class - only thing to do here is to open session to DB
     */
    public GroupDAO(){
	ss = SessionFactoryUtil.getSessionFactory().openSession();
    }
    /**
     * Method returning unfiltered List of all groups in the system
     * @return list of groups
     */
    public List<Group> getAllGroups(){
	ss.beginTransaction();
	List<Group> result = ss.createQuery( "from Group" ).list();
	ss.getTransaction().commit();
	return result;
    }
    /**
     * Method returning a Group object based on search by group name
     * @param groupid
     * @return group that matches the search criteria or null
     */
    public Group findGroup(String groupid){
	ss.beginTransaction();
	Group group =(Group) ss.get(Group.class, groupid);
	ss.getTransaction().commit();
	if (group != null){
	    return group;
	}else{
	    return null;
	}
    }
    /**
     * Method providing functionality for adding new Group
     * @param group
     * @return Returns true in case the creation succeeded, false in case it did not
     */
    public Boolean newGroup(Group group){
	if (findGroup(group.getGroupid()) == null){
	    ss.beginTransaction();
	    ss.save(group);
	    ss.getTransaction().commit();
	    return true;
	}else{
	    return false;
	}	
    }
    /**
     * Method providing functionality of updating an object 
     * @param group group that should be updated
     */
    public void updateGroup(Group group){
	ss.beginTransaction();
	ss.merge(group);
	ss.getTransaction().commit();
    }
    /**
     * method providing functionality for updating group name
     * @param oldName Old name of the group
     * @param newName New name of the group
     */
    public void updateGroupName(String oldName, String newName){
	ss.beginTransaction();
	String sQuery = "UPDATE groups SET groupid ='"+newName+"' WHERE groupid = '" + oldName + "'";
	SQLQuery query = ss.createSQLQuery(sQuery);
	query.executeUpdate();
	ss.getTransaction().commit();
    }
    /**
     * Method providing functionality for deleting a particular group
     * @param group group that is to be deleted
     */
    public void deleteGroup(Group group){
	ss.beginTransaction();
	ss.delete(group);
	ss.getTransaction().commit();
    }
    /**
     * Method for closing the session - it flushes database connection
     */
    public void closeSession(){
	ss.flush();
	ss.clear();
	ss.close();
    }
}
