package com.example.groracle.DAL.DAO;
import java.util.List;

import org.hibernate.SQLQuery;
import org.hibernate.Session;

import com.example.groracle.DAL.SessionFactoryUtil;
import com.example.groracle.Entity.Host;
/**
 * Class providing operations on top of Host object
 * @author tdosek
 * @version 1.1
 */
public class HostDAO {
    private Session ss = null;
    /**
     * Constructor of the class - only thing to do here is to open session to DB
     */
    public HostDAO(){
	ss = SessionFactoryUtil.getSessionFactory().openSession();
    }
    /**
     * Method returning list of all hosts
     * @return list of all hosts
     */
    public List<Host> getAllHosts(){
	ss.beginTransaction();
	List<Host> result = ss.createQuery( "from Host" ).list();
	ss.getTransaction().commit();
	return result;
    }
    /**
     * Method returning Host whose hostname matches the search criteria
     * @param host name of the host - hostname
     * @return Host that matches the criteria or null
     */
    public Host findHost(String host){
	ss.beginTransaction();
	Host resHost =(Host) ss.get(Host.class, host);
	ss.getTransaction().commit();
	if (resHost != null){
	    return resHost;
	}else{
	    return null;
	}
    }
    /**
     * Method returning list of hosts in certain group
     * @param groupname Name of the group
     * @return list of hosts in particular group
     */
    public List<Host> getHostsByGroup(String groupname){
	ss.beginTransaction();
	String sQuery = "SELECT * FROM hosts WHERE groupid = '"+groupname+"'";
	SQLQuery query = ss.createSQLQuery(sQuery).addEntity(Host.class);
	List<Host> result = query.list();
	ss.getTransaction().commit();
	return result;	
    }
    /**
     * Method creating new host in database
     * @param host Host to be created
     * @return Returns true in case the host got successfully created, otherwise false
     */
    public Boolean newHost(Host host){
	if (findHost(host.getHost()) == null){
	    ss.beginTransaction();
	    ss.save(host);
	    ss.getTransaction().commit();
	    return true;
	}else{
	    return false;
	}	
    }
    /**
     * Method providing functionality for host updating
     * @param host Host to be updated
     */
    public void updateHost(Host host){
	ss.beginTransaction();
	ss.merge(host);
	ss.getTransaction().commit();
    }
    /**
     * Method providing functionality for host deleting
     * @param host Host to be deleted
     */
    public void deleteHost(Host host){
	ss.beginTransaction();
	ss.delete(host);
	ss.getTransaction().commit();
    }
    /**
     * Method for closing the session - it flushes database connection
     */
    public void closeSession(){
	ss.flush();
	ss.clear();
	ss.close();
    }
}
