package com.example.groracle.DAL.DAO;
import java.util.List;

import org.hibernate.SQLQuery;
import org.hibernate.Session;

import com.example.groracle.DAL.SessionFactoryUtil;
import com.example.groracle.Entity.Input;
/**
 * Class providing operations on top of Input object
 * @author tdosek
 * @version 1.1
 */
public class InputDAO {
    private Session ss = null;
    /**
     * Constructor of the class - only thing to do here is to open session to DB
     */
    public InputDAO(){
	ss = SessionFactoryUtil.getSessionFactory().openSession();
    }
    /**
     * Method returning list of all Input entries   
     * @return All input entries
     */
    public List<Input> getAllInput(){
	ss.beginTransaction();
	List<Input> result = ss.createQuery( "from Input" ).list();
	ss.getTransaction().commit();
	return result;
    }
    /**
     * Method returning list of Input entries filtered by pattern
     * @param pattern Pattern name that is used as filter
     * @return List of Input entries which match the search criteria
     */
    public List<Input> getFilteredInput(String pattern){
	ss.beginTransaction();
	String sQuery = "SELECT * FROM input WHERE pattern = '"+pattern+"'";
	SQLQuery query = ss.createSQLQuery(sQuery).addEntity(Input.class);
	List<Input> result = query.list();
	ss.getTransaction().commit();
	return result;
    }
    /**
     * Method returning entries that were not blacklisted
     * @return List of entries that are not blacklisted
     */
    public List<Input> getNonBlacklisted(){
	ss.beginTransaction();
	String sQuery = "SELECT * FROM input LEFT JOIN pattern_blacklist ON "
		+ "input.pattern = pattern_blacklist.pattern WHERE pattern_blacklist.pattern IS NULL";
	SQLQuery query = ss.createSQLQuery(sQuery).addEntity(Input.class);
	List<Input> result = query.list();
	ss.getTransaction().commit();
	return result;
    }    
    /**
     * Method returning list of entries that are grouped by pattern
     * @return List of grouped entries
     */
    public List<Input> getNonBlacklistedGrouped(){
	ss.beginTransaction();
	String sQuery = "SELECT DISTINCT ON (input.pattern) * FROM input LEFT JOIN pattern_blacklist ON "
		+ "input.pattern = pattern_blacklist.pattern WHERE pattern_blacklist.pattern IS NULL";
	SQLQuery query = ss.createSQLQuery(sQuery).addEntity(Input.class);
	List<Input> result = query.list();
	ss.getTransaction().commit();
	return result;
    }
    /**
     * Method returning host matching search criteria
     * @param line Line that is unique identificatior of the input entry
     * @return Input entry matching the search crieria or null
     */
    public Input findInput(String line){
	ss.beginTransaction();
	Input resInput =(Input) ss.get(Input.class, line);
	ss.getTransaction().commit();
	if (resInput != null){
	    return resInput;
	}else{
	    return null;
	}
    }
    /**
     * Method providing functionality for creating new Input entry
     * @param input Input entry to be created
     * @return True in case the creation succeeded, otherwise false
     */
    public Boolean newInput(Input input){
	if (findInput(input.getLine()) == null){
	    ss.beginTransaction();
	    ss.save(input);
	    ss.getTransaction().commit();
	    return true;
	}else{
	    return false;
	}	
    }
    /**
     * Method providing functionality for updating single input line
     * @param input Input entry to be updated
     */
    public void updateInput(Input input){
	ss.beginTransaction();
	ss.merge(input);
	ss.getTransaction().commit();
    }
    /**
     * Method providing functionality for deleting Input entry
     * @param input Input entry to be deleted
     */
    public void deleteInput(Input input){
	ss.beginTransaction();
	ss.delete(input);
	ss.getTransaction().commit();
    }
    /**
     * Method for closing the session - it flushes database connection
     */
    public void closeSession(){
	ss.flush();
	ss.clear();
	ss.close();
    }
}
