package com.example.groracle.DAL.DAO;
import java.util.List;

import org.hibernate.SQLQuery;
import org.hibernate.Session;

import com.example.groracle.DAL.SessionFactoryUtil;
import com.example.groracle.Entity.Right;
/**
 * Class providing operations on top of Right object
 * @author tdosek
 * @version 1.1
 */
public class RightDAO {
    private Session ss = null;
    /**
     * Constructor of the class - only thing to do here is to open session to DB
     */
    public RightDAO(){
	ss = SessionFactoryUtil.getSessionFactory().openSession();
    }
    /**
     * Method returning list of all Rights
     * @return list of all rights
     */
    public List<Right> getAllRights(){
	ss.beginTransaction();
	List<Right> result = ss.createQuery( "from Right" ).list();
	ss.getTransaction().commit();
	return result;
    }
    /**
     * Method providing functionality for getting all rights for given user
     * @param username User whose rights should be returned
     * @return List of rights
     */
    public List<Right> getRightsByUser(String username){
	ss.beginTransaction();
	String sQuery = "SELECT * FROM rights WHERE usernames LIKE '%;"
		    +username+";%'";
	SQLQuery query = ss.createSQLQuery(sQuery).addEntity(Right.class);
	List<Right> result = query.list();
	ss.getTransaction().commit();
	ss.disableFilter("pattern_filter");
	return result;
    }
    /**
     * Method returning rights for given role
     * @param role name of role whose rights are to be displayed
     * @return list of rights
     */
    public List<Right> getRightsByRole(String role){
	ss.beginTransaction();
	String sQuery = "SELECT * FROM rights WHERE roles LIKE '%;"
		    +role+";%'";
	SQLQuery query = ss.createSQLQuery(sQuery).addEntity(Right.class);
	List<Right> result = query.list();
	ss.getTransaction().commit();
	ss.disableFilter("pattern_filter");
	return result;
    }
    /**
     * Method returning right that matches the permission given
     * @param permission permission string that is being searched for
     * @return Right that matches search criteria
     */
    public Right findRight(String permission){
	ss.beginTransaction();
	Right resRight =(Right) ss.get(Right.class, permission);
	ss.getTransaction().commit();
	if (resRight != null){
	    return resRight;
	}else{
	    return null;
	}
    }
    /**
     * Method providing functionality for creating new right
     * @param right Right to be created
     * @return Return true in case the creation succeeds, otherwise false
     */
    public Boolean newRight(Right right){
	if (findRight(right.getPermission()) == null){
	    ss.beginTransaction();
	    ss.save(right);
	    ss.getTransaction().commit();
	    return true;
	}else{
	    return false;
	}	
    }
    /**
     * Method providing functionality for updating right
     * @param right Right to be updated
     */
    public void updateRight(Right right){
	ss.beginTransaction();
	ss.merge(right);
	ss.getTransaction().commit();
    }
    /**
     * Method providing functionality for deleting right
     * @param right Right to be deleted
     */
    public void deleteRight(Right right){
	ss.beginTransaction();
	ss.delete(right);
	ss.getTransaction().commit();
    }
    /**
     * Method for closing the session - it flushes database connection
     */
    public void closeSession(){
	ss.flush();
	ss.clear();
	ss.close();
    }
}
