package com.example.groracle.DAL.DAO;
import java.util.List;

import org.hibernate.SQLQuery;
import org.hibernate.Session;

import com.example.groracle.DAL.SessionFactoryUtil;
import com.example.groracle.Entity.Role;
/**
 * Class providing operations on top of Role object
 * @author tdosek
 * @version 1.1
 */
public class RoleDAO {
    private Session ss = null;
    /**
     * Constructor of the class - only thing to do here is to open session to DB
     */
    public RoleDAO(){
	ss = SessionFactoryUtil.getSessionFactory().openSession();
    }
    /**
     * Method returning List of Roles
     * @return list of roles
     */
    public List<Role> getAllRoles(){
	ss.beginTransaction();
	List<Role> result = ss.createQuery( "from Role" ).list();
	ss.getTransaction().commit();
	return result;
    }
    /**
     * Method returning role that matches role name
     * @param role Role name we search for
     * @return role that is being found or null
     */
    public Role findRole(String role){
	ss.beginTransaction();
	Role resRole =(Role) ss.get(Role.class, role);
	ss.getTransaction().commit();
	if (resRole != null){
	    return resRole;
	}else{
	    return null;
	}
    }
    /**
     * Method returning providing funtionality for creating new Role
     * @param role Role that is to be created
     * @return true in case the role creation succeeded, otherwise false
     */
    public Boolean newRole(Role role){
	if (findRole(role.getRole()) == null){
	    ss.beginTransaction();
	    ss.save(role);
	    ss.getTransaction().commit();
	    return true;
	}else{
	    return false;
	}	
    }
    /**
     * Method providing functionality for updating Role entry
     * @param role Role to be updated
     */
    public void updateRole(Role role){
	ss.beginTransaction();
	ss.merge(role);
	ss.getTransaction().commit();
    }
    /**
     * Method providing functionality for updating role name
     * @param oldName Old name of the role
     * @param newName New name for the role
     */
    public void updateRoleName(String oldName, String newName){
	ss.beginTransaction();
	String sQuery = "UPDATE roles SET role ='"+newName+"' WHERE role = '" + oldName + "'";
	SQLQuery query = ss.createSQLQuery(sQuery);
	query.executeUpdate();
	ss.getTransaction().commit();
    }
    /**
     * Method providing functionality for deleting a role
     * @param role Role that is to be deleted
     */
    public void deleteRole(Role role){
	ss.beginTransaction();
	ss.delete(role);
	ss.getTransaction().commit();
    }
    /**
     * Method for closing the session - it flushes database connection
     */
    public void closeSession(){
	ss.flush();
	ss.clear();
	ss.close();
    }
}
