package com.example.groracle.DAL.DAO;
import java.util.List;

import org.hibernate.SQLQuery;
import org.hibernate.Session;

import com.example.groracle.DAL.SessionFactoryUtil;
import com.example.groracle.Entity.User;
/**
 * Class providing operations on top of User object
 * @author tdosek
 * @version 1.1
 */
public class UserDAO {
    private Session ss = null;
    /**
     * Constructor of the class - only thing to do here is to open session to DB
     */
    public UserDAO(){
	ss = SessionFactoryUtil.getSessionFactory().openSession();
    }
    /**
     * Method returning all Users
     * @return all users
     */
    public List<User> getAllUsers(){
   	ss.beginTransaction();
   	List<User> result = ss.createQuery( "from User" ).list();
   	ss.getTransaction().commit();
   	return result;
    }
    /**
     * Method returning a user that matches username being searched for
     * @param username Username of the user 
     * @return User that is being looked for or null if the search fails
     */
    public User findUser(String username){
   	ss.beginTransaction();
   	User resUser =(User) ss.get(User.class, username);
   	ss.getTransaction().commit();
   	if (resUser != null){
   	    return resUser;
   	}else{
   	    return null;
   	}
   }
   /**
    * Method providing functionality for creating new user
    * @param user User to be created
    * @return True in case that user creation completed successfully, otherwise false
    */
   public Boolean newUser(User user){
   	if (findUser(user.getUsername()) == null){
   	    ss.beginTransaction();
   	    ss.save(user);
   	    ss.getTransaction().commit();
   	    return true;
   	}else{
   	    return false;
   	}	
   }
   /**
    * Method providing functionality for updating User entry
    * @param user User to be updated
    */
   public void updateUser(User user){
   	ss.beginTransaction();
   	ss.merge(user);
   	ss.getTransaction().commit();
   }
   /**
    * Method providing functionality of mass update of roles
    * @param oldRole Role name that has to be changed
    * @param newRole New role name
    */
   public void onRoleUpdate(String oldRole, String newRole){
       if (newRole == null){
	   ss.beginTransaction();
	   String sQuery = "UPDATE users SET role ='' WHERE role = '" + oldRole + "'";
	   SQLQuery query = ss.createSQLQuery(sQuery);
	   query.executeUpdate();
	   ss.getTransaction().commit();
       }else{
	   ss.beginTransaction();
	   String sQuery = "UPDATE users SET role ='"+newRole+"' WHERE role = '" + oldRole + "'";
	   SQLQuery query = ss.createSQLQuery(sQuery);
	   query.executeUpdate();
	   ss.getTransaction().commit();
       }
   }
   /**
    * Method providing functionality for deleting a user
    * @param user user to be deleted
    */
   public void deleteUser(User user){
   	ss.beginTransaction();
   	ss.delete(user);
   	ss.getTransaction().commit();
   }
   /**
    * Method for closing the session - it flushes database connection
    */
   public void closeSession(){
       ss.flush();
       ss.clear();
       ss.close();
   }
}
