package com.example.groracle;

import com.example.groracle.Brokers.CommonDatabaseBroker;
import com.example.groracle.Brokers.CommonPermissionsBroker;
import com.example.groracle.Views.AboutView;
import com.example.groracle.Views.AddGroupView;
import com.example.groracle.Views.BlacklistView;
import com.example.groracle.Views.ConfirmedView;
import com.example.groracle.Views.EditGroupView;
import com.example.groracle.Views.EditHostGroupView;
import com.example.groracle.Views.EditHostView;
import com.example.groracle.Views.EditRoleGroupsView;
import com.example.groracle.Views.EditRolePermissionsView;
import com.example.groracle.Views.EditRoleView;
import com.example.groracle.Views.EditUserGroupsView;
import com.example.groracle.Views.EditUserPermissionsView;
import com.example.groracle.Views.EditUserRoleView;
import com.example.groracle.Views.EditUserView;
import com.example.groracle.Views.GrokBlacklistFileUploadView;
import com.example.groracle.Views.GrokFileUploadView;
import com.example.groracle.Views.NewHostView;
import com.example.groracle.Views.NewRoleView;
import com.example.groracle.Views.NewUserView;
import com.example.groracle.Views.RemoveUserView;
import com.example.groracle.Views.ResetPasswordView;
import com.example.groracle.Views.ResetUsersPasswordView;
import com.example.groracle.Views.StatisticsView;
import com.vaadin.navigator.Navigator;
import com.vaadin.server.ClassResource;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Image;
import com.vaadin.ui.Label;
import com.vaadin.ui.MenuBar;
import com.vaadin.ui.Panel;
import com.vaadin.ui.UI;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.MenuBar.MenuItem;
/**
 * Class representing menu inmplementation and relevant handling of menu items
 * @author tdosek
 * @version 1.0
 */
public class Menu extends Panel {
    private VerticalLayout header = new VerticalLayout();
    private HorizontalLayout imgPanel = new HorizontalLayout();
    private Image headerImg = new Image(null, new ClassResource("groracle.gif"));
    private Label headerLabel = new Label("Groracle... The source of wisdom for your Grok",Label.CONTENT_RAW);
    private final com.vaadin.ui.MenuBar menu = new com.vaadin.ui.MenuBar();
    private Navigator navigator = null;
    private String loggedInUser = null;
    private String userRole = null;
    private CommonPermissionsBroker pb = new CommonPermissionsBroker();
    /**
     * Constructor of the class - specific implementation in cosntructor - load user credentials and roles
     * @param navigator variable representing UI navigator class which handles UI element switching
     */
    public Menu(Navigator navigator){
	this.navigator = navigator;
	this.loggedInUser = ((NavigatorUI)UI.getCurrent()).getLoggedInUser();
	CommonDatabaseBroker db = new CommonDatabaseBroker("SELECT role FROM users WHERE username = '"+loggedInUser+"';");
	db.executeSelect();
	userRole = db.getList().get(0).get("role").toString();
    }
    /**
     * Method initializing menu layout and header of the menu
     */
    public void initMenu(){
	// Display header
	setContent(header);	    
	header.addComponent(imgPanel);
	header.setWidth("100%");
	header.setHeight("100%");
	imgPanel.setStyleName("backColor");
	imgPanel.setWidth("100%");
	header.setWidth("100%");
	imgPanel.addComponent(headerImg);
	headerImg.setWidth("150px");
	headerImg.setHeight("113px");
	imgPanel.addComponent(headerLabel);
	headerLabel.setStyleName("headerLabel");
	header.addComponent(menu);
	menu.setWidth("100%");
	
	if (pb.validateUserPermission("EditPattern", loggedInUser) || pb.validateRolePermission("EditPattern", userRole)){
	// Patterns menu
	MenuItem patterns = menu.addItem("Patterns", null);
	patterns.addItem("Unconfirmed/nonblacklisted patterns", new MenuBar.Command() {
	        
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo("/");	    	
	    }
	});
	patterns.addItem("Blacklisted patterns", new MenuBar.Command() {
			        
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(BlacklistView.NAME);	    	
	    }
	});
	patterns.addItem("Confirmed patterns", new MenuBar.Command() {
	    
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(ConfirmedView.NAME);		
	    }
	});
	}
	
	if (pb.validateUserPermission("EditGrok", loggedInUser) || pb.validateRolePermission("EditGrok", userRole)){
	//  Grok menu
	MenuItem grok = menu.addItem("Grok", null);
	grok.addItem("Actual grok file upload", new MenuBar.Command() {
	    
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(GrokFileUploadView.NAME);		
	    }
	});
	grok.addItem("Actual grok blacklist file upload", new MenuBar.Command() {
	    
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(GrokBlacklistFileUploadView.NAME);
	    }
	});
	}
	
	if (pb.validateUserPermission("EditUser", loggedInUser) || pb.validateRolePermission("EditUser", userRole)){
	//  Users menu
	MenuItem users = menu.addItem("Users", null);
	users.addItem("Add user", new MenuBar.Command() {
	    
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(NewUserView.NAME);		
	    }
	});
	users.addItem("Remove user", new MenuBar.Command() {
	    
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(RemoveUserView.NAME);
	    }
	});
	users.addItem("Edit user", new MenuBar.Command() {
	    
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(EditUserView.NAME);		
	    }
	});
	users.addItem("Edit user permissions", new MenuBar.Command() {
	    
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(EditUserPermissionsView.NAME);	
	    }
	});
	users.addItem("Reset user's password", new MenuBar.Command() {
	    
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(ResetUsersPasswordView.NAME);
	    }
	});
	}
	
	if (pb.validateUserPermission("EditHosts", loggedInUser) || pb.validateRolePermission("EditHosts", userRole)){
	// Hosts menu
	MenuItem hosts = menu.addItem("Hosts", null);
	hosts.addItem("New host", new MenuBar.Command() {
	    
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(NewHostView.NAME);
	    }
	});
	hosts.addItem("Edit/remove host", new MenuBar.Command() {
	    
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(EditHostView.NAME);
	    }
	});
	}
	
	if (pb.validateUserPermission("EditGroup", loggedInUser) || pb.validateRolePermission("EditGroup", userRole)){
	//  Groups menu
	MenuItem groups = menu.addItem("Groups", null);
	groups.addItem("Add group", new MenuBar.Command() {
	    
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(AddGroupView.NAME);		
	    }
	});
	groups.addItem("Edit/remove group", new MenuBar.Command() {
	    
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(EditGroupView.NAME);		
	    }
	});
	groups.addItem("Edit/remove user's groups", new MenuBar.Command() {
	    
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(EditUserGroupsView.NAME);
	    }
	});
	groups.addItem("Edit/remove role's groups", new MenuBar.Command() {
	    
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(EditRoleGroupsView.NAME);		
	    }
	});
	groups.addItem("Edit host groups", new MenuBar.Command() {
	    
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(EditHostGroupView.NAME);
	    }
	});
	}
	
	if (pb.validateUserPermission("EditRole", loggedInUser) || pb.validateRolePermission("EditRole", userRole)){
	// Roles menu
	MenuItem roles = menu.addItem("Roles", null);
	roles.addItem("New role", new MenuBar.Command() {
	    
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(NewRoleView.NAME);		
	    }
	});
	roles.addItem("Edit/remove role", new MenuBar.Command() {
	    
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(EditRoleView.NAME);
	    }
	});
	roles.addItem("Edit user roles", new MenuBar.Command() {
	    
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(EditUserRoleView.NAME);		
	    }
	});
	roles.addItem("Edit role permissions", new MenuBar.Command() {
	    
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(EditRolePermissionsView.NAME);
		
	    }
	});
	}
	
	//  Statistics menu
	MenuItem statistics = menu.addItem("Statistics", new MenuBar.Command() {
	    
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(StatisticsView.NAME);		
	    }
	});
	
	// Settings menu
	MenuItem settings = menu.addItem("Settings", null);
	settings.addItem("Reset password", new MenuBar.Command() {
	    
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(ResetPasswordView.NAME);
	    }
	});
	
	//  About menu
	MenuItem about = menu.addItem("About", new MenuBar.Command() {
	    
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
		navigator.navigateTo(AboutView.NAME);
	    }
	});
	
	//  Logout menu
	MenuItem logout = menu.addItem("Log me out", new MenuBar.Command() {
			        
	    @Override
	    public void menuSelected(MenuItem selectedItem) {
	       	((NavigatorUI)UI.getCurrent()).setLoggedInUser(null);
	       	navigator.navigateTo("/");
	    }
	});
    }
}