package com.example.groracle;

import javax.servlet.annotation.WebServlet;

import com.example.groracle.Views.AboutView;
import com.example.groracle.Views.AddGroupView;
import com.example.groracle.Views.BlacklistView;
import com.example.groracle.Views.ConfirmedView;
import com.example.groracle.Views.EditGroupView;
import com.example.groracle.Views.EditHostGroupView;
import com.example.groracle.Views.EditHostView;
import com.example.groracle.Views.EditRoleGroupsView;
import com.example.groracle.Views.EditRolePermissionsView;
import com.example.groracle.Views.EditRoleView;
import com.example.groracle.Views.EditUserGroupsView;
import com.example.groracle.Views.EditUserPermissionsView;
import com.example.groracle.Views.EditUserRoleView;
import com.example.groracle.Views.EditUserView;
import com.example.groracle.Views.GrokBlacklistFileUploadView;
import com.example.groracle.Views.GrokFileUploadView;
import com.example.groracle.Views.LoginView;
import com.example.groracle.Views.MainView;
import com.example.groracle.Views.NewHostView;
import com.example.groracle.Views.NewRoleView;
import com.example.groracle.Views.NewUserView;
import com.example.groracle.Views.RemoveUserView;
import com.example.groracle.Views.ResetPasswordView;
import com.example.groracle.Views.ResetUsersPasswordView;
import com.example.groracle.Views.StatisticsView;
import com.vaadin.annotations.Theme;
import com.vaadin.annotations.VaadinServletConfiguration;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.ViewChangeListener;
import com.vaadin.server.VaadinRequest;
import com.vaadin.server.VaadinServlet;
import com.vaadin.ui.UI;

@SuppressWarnings("serial")
@Theme("groracle")
/**
 * Main class of the project which is responsible for providing all classes and serving as servlet.
 * We use Servlet API 3.0 for handling the URI fragments and other functionality
 * @author tdosek
 */
public class NavigatorUI extends UI {
    // Serve the servlet, and relevant widgetsets
    @WebServlet(value = "/*", asyncSupported = true)
    @VaadinServletConfiguration(productionMode = false, ui = NavigatorUI.class,
	   widgetset="com.example.groracle.widgetset.GroracleWidgetSet")
    /**
     * Servlet serving whole application
     * @author tdosek
     */
    public static class Servlet extends VaadinServlet {
    }
    protected Navigator navigator;
    //  Logged in user - if null, we only allow to see the login view
    private String loggedInUser = null;

    @Override
    /*
     * (non-Javadoc)
     * @see com.vaadin.ui.UI#init(com.vaadin.server.VaadinRequest)
     */
    public void init(VaadinRequest request) {
        // Create Navigator, make it control the ViewDisplay
        navigator = new Navigator(this, this);

        // Add some Views
        navigator.addView(MainView.NAME, new MainView(navigator));// no fragment
        navigator.addView(BlacklistView.NAME, new BlacklistView(navigator));
        navigator.addView(ConfirmedView.NAME, new ConfirmedView(navigator));
        navigator.addView(ResetPasswordView.NAME, new ResetPasswordView(navigator));
        navigator.addView(AboutView.NAME, new AboutView(navigator));
        navigator.addView(NewUserView.NAME, new NewUserView(navigator));
        navigator.addView(RemoveUserView.NAME, new RemoveUserView(navigator));
        navigator.addView(EditUserView.NAME, new EditUserView(navigator));
        navigator.addView(NewRoleView.NAME, new NewRoleView(navigator));
        navigator.addView(EditRoleView.NAME, new EditRoleView(navigator));
        navigator.addView(EditUserRoleView.NAME, new EditUserRoleView(navigator));
        navigator.addView(EditRolePermissionsView.NAME, new EditRolePermissionsView(navigator));
        navigator.addView(EditUserPermissionsView.NAME, new EditUserPermissionsView(navigator));
        navigator.addView(AddGroupView.NAME, new AddGroupView(navigator));
        navigator.addView(EditGroupView.NAME, new EditGroupView(navigator));
        navigator.addView(EditUserGroupsView.NAME, new EditUserGroupsView(navigator));
        navigator.addView(EditRoleGroupsView.NAME, new EditRoleGroupsView(navigator));
        navigator.addView(NewHostView.NAME, new NewHostView(navigator));
        navigator.addView(EditHostView.NAME, new EditHostView(navigator));
        navigator.addView(EditHostGroupView.NAME, new EditHostGroupView(navigator));
        navigator.addView(ResetUsersPasswordView.NAME, new ResetUsersPasswordView(navigator));
        navigator.addView(GrokFileUploadView.NAME, new GrokFileUploadView(navigator));
        navigator.addView(GrokBlacklistFileUploadView.NAME, new GrokBlacklistFileUploadView(navigator));
        navigator.addView(StatisticsView.NAME, new StatisticsView(navigator));
        // We'll handle permissions with a listener here, you could also do
        // that in the View itself.

        navigator.addViewChangeListener(new ViewChangeListener() {

	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.navigator.ViewChangeListener#beforeViewChange(com.vaadin.navigator.ViewChangeListener.ViewChangeEvent)
	     */
	    public boolean beforeViewChange(ViewChangeEvent event) {
	        if (((NavigatorUI)UI.getCurrent()).getLoggedInUser() == null) {
	            // Show to LoginView instead, pass intended view
	            String fragmentAndParameters = event.getViewName();
	            if (event.getParameters() != null) {
	                fragmentAndParameters += "/";
	                fragmentAndParameters += event.getParameters();
	            }
	            navigator.getDisplay().showView(new LoginView(navigator,
	                    fragmentAndParameters));
	            return false;
	        } else return true;
	    }
	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.navigator.ViewChangeListener#afterViewChange(com.vaadin.navigator.ViewChangeListener.ViewChangeEvent)
	     */
	    public void afterViewChange(ViewChangeEvent event) {

	    }          
	});
    }
    /**
     * Getter method which is supposed to return username of currently logged-in user.
     * @return username
     */
    public String getLoggedInUser(){
         return loggedInUser;
    } 
    /**
     * Setter method which sets username of user that is logged in
     * @param user Username of user that logged in
     */
    public void setLoggedInUser(String user){
         loggedInUser = user;
   }
    
}
