package com.example.groracle.Views;

import java.util.List;

import com.example.groracle.Menu;
import com.example.groracle.DAL.DAO.HostDAO;
import com.example.groracle.Entity.Host;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.fieldgroup.FieldGroup;
import com.vaadin.data.util.IndexedContainer;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener.ViewChangeEvent;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.FormLayout;
import com.vaadin.ui.Notification;
import com.vaadin.ui.Panel;
import com.vaadin.ui.Table;
import com.vaadin.ui.TextArea;
import com.vaadin.ui.TextField;
import com.vaadin.ui.VerticalLayout;
/**
 * Class representing UI implementation of host editing
 * @author tdosek
 * @version 1.0
 */
public class EditHostView extends Panel implements View{
    public static final String NAME = "EditHost";
    private Navigator navigator = null;
    private VerticalLayout header = new VerticalLayout();
    // Initialization of variables for the view
    private IndexedContainer hostlist = null;
    private Table hostList = new Table();
    private static String hostname = "Host name";
    private static String hostip = "Host IP address";
    private static String hostdesc = "Host description";
    private FormLayout hostlistEditorLayout = new FormLayout();
    private FieldGroup editorFields = new FieldGroup();
    private final TextField hostF = new TextField(hostname);
    private final TextField hostIpF = new TextField(hostip);
    private final TextArea hostDescF = new TextArea(hostdesc);
    // Initialization of variables for item selection
    private String selectedHost = null;
    private Object lineId = null;
    /**
     * Constructor of the class
     * @param navigator variable representing UI navigator class which handles UI element switching
     */
    public EditHostView(Navigator navigator){
	this.navigator = navigator;
    }
    /**
     * Initialization method which handles composition of UI for given class
     */
    private void init(){
	setContent(header);
	header.removeAllComponents();
	Menu menu = new Menu(navigator);
	menu.initMenu();
	header.addComponent(menu);
	hostlist = createHostListDatasource();
	initListLayout();
	initEditor();
	initList();
    }
    /**
     * Method responsible for creating data source for given table
     * @return data source for table
     */
    private static IndexedContainer createHostListDatasource() {
	IndexedContainer ic = new IndexedContainer();
	for (String p : new String[]{hostname,hostip,hostdesc}){
	    ic.addContainerProperty(p, String.class, "");
	}
	HostDAO hDal = new HostDAO();
	List<Host> hosts = hDal.getAllHosts();
	for (Host i: hosts){
	    Object id = ic.addItem();
	    String hostNameAdd = i.getHost();
	    String hostIpAdd = i.getIp();
	    String hostDescAdd = i.getDescription();
	    ic.getContainerProperty(id, hostname).setValue(hostNameAdd);
	    ic.getContainerProperty(id, hostip).setValue(hostIpAdd);
	    ic.getContainerProperty(id, hostdesc).setValue(hostDescAdd);
	}
	hDal.closeSession();
	return ic;
    }
    /**
     * Method intializing layout of the editor
     */
    private void initListLayout(){
	hostList.setStyleName("v-table-strong");
	final VerticalLayout leftLayout = new VerticalLayout();
	header.addComponent(leftLayout);	    
	header.addComponent(hostlistEditorLayout);
	    
	leftLayout.addComponent(hostList);
	hostList.setHeight("100%");
	hostList.setWidth("100%");
	leftLayout.setSizeFull();
	leftLayout.setExpandRatio(hostList, 1);
    }
    /**
     * Method initializing editor layout and relevant elements handling
     */
    private void initEditor(){
	VerticalLayout editor = new VerticalLayout();
	editor.setStyleName("loginEditor");
	header.addComponent(editor);
	
	if (hostlistEditorLayout.getComponentCount() == 0){
	    hostlistEditorLayout.addComponent(hostF);
	    editorFields.bind(hostF, hostname);
	    hostF.setReadOnly(true);
	    hostlistEditorLayout.addComponent(hostIpF);
	    editorFields.bind(hostIpF, hostip);
	    hostlistEditorLayout.addComponent(hostDescF);
	    editorFields.bind(hostDescF, hostdesc);
	
	    Button editHostB = new Button("Save change");
	    hostlistEditorLayout.addComponent(editHostB);
	    editHostB.addClickListener(new Button.ClickListener() {
	        
	        @Override
	        /*
	         * (non-Javadoc)
	         * @see com.vaadin.ui.Button.ClickListener#buttonClick(com.vaadin.ui.Button.ClickEvent)
	         */
	        public void buttonClick(ClickEvent event) {
	            if (!(selectedHost==null)){
	        	if (!(hostF.getValue()==null) && !(hostF.getValue().equals("")) && !(hostIpF.getValue().equals("")) && !(hostIpF.getValue()==null)){
	        	    HostDAO hDal = new HostDAO();
	        	    Host editedHost = hDal.findHost(selectedHost);
	        	    editedHost.setDescription(hostDescF.getValue());
	        	    editedHost.setIp(hostIpF.getValue());
	        	    hDal.updateHost(editedHost);
	        	    hDal.closeSession();
	        	    hostDescF.setValue("");
	        	    hostIpF.setValue("");
	        	    Notification.show("Host successfully added");
	        	}else Notification.show("Failed to update host","Host name and IP can't be empty",Notification.TYPE_ERROR_MESSAGE);
	            }else Notification.show("Failed to update host","You have to selected and edit a host first",Notification.TYPE_ERROR_MESSAGE);
	            init();
	        }
	    });
	    
	    Button deleteHostB = new Button("Delete host");
	    hostlistEditorLayout.addComponent(deleteHostB);
	    deleteHostB.addClickListener(new Button.ClickListener() {
	        
	        @Override
	        /*
	         * (non-Javadoc)
	         * @see com.vaadin.ui.Button.ClickListener#buttonClick(com.vaadin.ui.Button.ClickEvent)
	         */
	        public void buttonClick(ClickEvent event) {
	            if(!(selectedHost==null)&&!(selectedHost.equals(""))){
	        	HostDAO hDal = new HostDAO();
	        	Host deletedHost = hDal.findHost(selectedHost);
	        	hDal.deleteHost(deletedHost);
	        	hDal.closeSession();
	        	hostF.setReadOnly(false);
	        	hostF.setValue("");
	        	hostF.setReadOnly(true);
	        	hostIpF.setValue("");
	        	hostDescF.setValue("");
	        	Notification.show("Host succesfully deted");
	        	init();
	            }else Notification.show("Failed to deete host","You have to select a host first",Notification.TYPE_ERROR_MESSAGE);
	        }
	    });
	}
    }
    /**
     * Method initializing table and mapping it to relevant data source
     */
    private void initList(){
	hostList.setContainerDataSource(hostlist);
	hostList.setVisibleColumns(new String[]{hostname,hostip,hostdesc});
	hostList.setSelectable(true);
	hostList.setImmediate(true);
	hostList.addValueChangeListener(new Property.ValueChangeListener() {
	    
	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.data.Property.ValueChangeListener#valueChange(com.vaadin.data.Property.ValueChangeEvent)
	     */
	    public void valueChange(ValueChangeEvent event) {
		lineId = hostList.getValue();
		if(lineId != null){
		    editorFields.setItemDataSource(hostList.getItem(lineId));
	    	    selectedHost = hostList.getItem(lineId).getItemProperty(hostname).getValue().toString();
	    	    editorFields.getField(hostname).setReadOnly(true);
		}
	    }
	});
    }
    @Override
    /*
     * (non-Javadoc)
     * @see com.vaadin.navigator.View#enter(com.vaadin.navigator.ViewChangeListener.ViewChangeEvent)
     */
    public void enter(ViewChangeEvent event) {
	init();
    }

}
