package com.example.groracle.Views;

import java.util.List;

import com.example.groracle.Menu;
import com.example.groracle.DAL.DAO.UserDAO;
import com.example.groracle.Entity.User;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.fieldgroup.FieldGroup;
import com.vaadin.data.util.IndexedContainer;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener.ViewChangeEvent;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.FormLayout;
import com.vaadin.ui.Notification;
import com.vaadin.ui.Panel;
import com.vaadin.ui.Table;
import com.vaadin.ui.TextField;
import com.vaadin.ui.VerticalLayout;
/**
 * Class representing UI implementation of user editing fucntions
 * @author tdosek
 * @version 1.0
 */
public class EditUserView extends Panel implements View{
    public static final String NAME = "EditUser";
    private Navigator navigator = null;
    // Initializing variables for view
    private VerticalLayout header = new VerticalLayout();
    // Initializing variables for the view 
    private IndexedContainer userlist = null;
    private Button editFromUserlistButton = new Button("Save the change");
    private FormLayout userlistEditorLayout = new FormLayout();
    private static String username = "User name";
    private static String email = "E-mail";
    private static String description = "Description";
    private FieldGroup editorFields = new FieldGroup();
    private static final String[] userFieldNames = new String[] {username,email,description};
    private Table userList = new Table();    
    // 	Variables for storing pattern selected in UI
    private String selectedUsername = null;
    private Object lineId = null;
    /**
     * Constructor of the class
     * @param navigator variable representing UI navigator class which handles UI element switching 
     */
    public EditUserView(Navigator navigator){
	this.navigator = navigator;
    } 
    /**
     * Initialization method which handles composition of UI for given class
     */
    private void init(){
	setContent(header);
	header.removeAllComponents();
	Menu menu = new Menu(navigator);
	menu.initMenu();
	header.addComponent(menu);
	selectedUsername = null;
	lineId = null;
	userlist = createUserlistDatasource();
	initListLayout();
	initUserListEditor();
	initList();	
    }
    /**
     * Initialization method for table layout of the editor
     */
    private void initListLayout(){
	userList.setStyleName("v-table-strong");
	final VerticalLayout leftLayout = new VerticalLayout();
	header.addComponent(leftLayout);	    
	header.addComponent(userlistEditorLayout);
	    
	leftLayout.addComponent(userList);
	userList.setHeight("100%");
	leftLayout.setSizeFull();
	leftLayout.setExpandRatio(userList, 1);
    }
    /**
     * Initialization method for editor layout of the editor
     */
    private void initUserListEditor(){
	userList.setWidth("100%");
	if (userlistEditorLayout.getComponentCount() == 0){
	    for (String fieldName: userFieldNames){
		TextField field = new TextField(fieldName);
		userlistEditorLayout.addComponent(field);
		if (fieldName == username) field.setReadOnly(true);
		field.setWidth("100%");
		field.setImmediate(true);
		editorFields.bind(field, fieldName);
	    }
	    
	    userlistEditorLayout.addComponent(editFromUserlistButton);
	    editFromUserlistButton.addClickListener(new ClickListener() {
		    
	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.ui.Button.ClickListener#buttonClick(com.vaadin.ui.Button.ClickEvent)
	     */
	    public void buttonClick(ClickEvent event) {
		Object line = userList.getValue();
		if (selectedUsername.equals(editorFields.getField(username).getValue().toString())){
		    if (line != null){
			UserDAO uDal = new UserDAO();
			User selUser = uDal.findUser(selectedUsername);
			selUser.setDescription(editorFields.getField(description).getValue().toString());
			selUser.setEmail(editorFields.getField(email).getValue().toString());
			uDal.updateUser(selUser);
			uDal.closeSession();
			Notification.show("Changes successfully saved");
			init();
		    }
		}else Notification.show("Username cannot be changed!",Notification.TYPE_ERROR_MESSAGE);
	    }
	});	   
    	}
    }
    /**
     * Method for inititalization of table and its connection to relevant datasource
     */
    private void initList(){
	userList.setContainerDataSource(userlist);
	userList.setVisibleColumns(userFieldNames);
	userList.setSelectable(true);
	userList.setImmediate(true);
	userList.addValueChangeListener(new Property.ValueChangeListener() {
	        
	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.data.Property.ValueChangeListener#valueChange(com.vaadin.data.Property.ValueChangeEvent)
	     */
	    public void valueChange(ValueChangeEvent event) {
		lineId = userList.getValue();
	    	if (lineId != null){
	    	    editorFields.setItemDataSource(userList.getItem(lineId));
	    	    selectedUsername = userList.getItem(lineId).getItemProperty(username).getValue().toString();
	    	    editorFields.getField(username).setReadOnly(true);
	 	}	
	    }
	});
	userList.setBuffered(false);
    }
    /**
     * Method for initialization of a datasource for a table
     * @return data source
     */
    private static IndexedContainer createUserlistDatasource(){
	    // 	Loading database values into an Indexed container for further usage by UI.
		IndexedContainer ic = new IndexedContainer();
		for (String p : userFieldNames){
		    ic.addContainerProperty(p, String.class, "");
		}
		UserDAO uDal = new UserDAO();
		List<User> allUsers = uDal.getAllUsers();
		for (User i: allUsers){
		    Object id = ic.addItem();
		    String userNameAdd = i.getUsername();
		    String emailAdd = i.getEmail();
		    String descAdd = i.getDescritpion();
		    ic.getContainerProperty(id, username).setValue(userNameAdd);
		    ic.getContainerProperty(id, email).setValue(emailAdd);
		    ic.getContainerProperty(id, description).setValue(descAdd);
		}
		uDal.closeSession();
		return ic;
	    }
    @Override
    /*
     * (non-Javadoc)
     * @see com.vaadin.navigator.View#enter(com.vaadin.navigator.ViewChangeListener.ViewChangeEvent)
     */
    public void enter(ViewChangeEvent event) {
	init();	
    }

}
