package com.example.groracle.Views;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.List;

import com.example.groracle.Menu;
import com.example.groracle.Brokers.CommonFileUploadBroker;
import com.example.groracle.Brokers.ConfigLoader;
import com.example.groracle.Brokers.GrokFileGenerator;
import com.example.groracle.DAL.DAO.GroupDAO;
import com.example.groracle.DAL.DAO.HostDAO;
import com.example.groracle.Entity.Group;
import com.example.groracle.Entity.Host;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener.ViewChangeEvent;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.HorizontalSplitPanel;
import com.vaadin.ui.Label;
import com.vaadin.ui.Notification;
import com.vaadin.ui.OptionGroup;
import com.vaadin.ui.Panel;
import com.vaadin.ui.Select;
import com.vaadin.ui.TextArea;
import com.vaadin.ui.VerticalLayout;
/**
 * Class representing UI implementation of Black list file upload
 * @author tdosek
 * @version 1.0
 */
public class GrokBlacklistFileUploadView extends Panel implements View {
    public static final String NAME = "GrokBlacklistFileUpload";
    private Navigator navigator = null;
    // Initializing variables for view
    private VerticalLayout header = new VerticalLayout();
    private TextArea grokContent = new TextArea();
    /**
     * Constructor of the class
     * @param navigator variable representing UI navigator class which handles UI element switching 
     */
    public GrokBlacklistFileUploadView(Navigator navigator){
	this.navigator = navigator;
    }

    private void init(){
	setContent(header);
	header.removeAllComponents();
	Menu menu = new Menu(navigator);
	menu.initMenu();
	header.addComponent(menu);
	grokContent.setReadOnly(false);
	grokContent.setValue("");
	initEditor();
    }
    
    @SuppressWarnings("deprecation")
    /**
     * Method initialiazing editor fields and their relevant handlings
     */
    private void initEditor(){
	header.setSizeFull();
	HorizontalLayout editor = new HorizontalLayout();
	editor.setWidth("100%");
	header.addComponent(editor);
	
	HorizontalSplitPanel hsplit = new HorizontalSplitPanel();
	hsplit.setHeight("100%");
	hsplit.setWidth("100%");
	editor.addComponent(hsplit);
	VerticalLayout leftLayout = new VerticalLayout();
	VerticalLayout rightLayout = new VerticalLayout();
	
	hsplit.setFirstComponent(leftLayout);
	hsplit.setSecondComponent(rightLayout);
	// Generate future file content and display it
	leftLayout.addComponent(grokContent);
	grokContent.setWidth("100%");
	grokContent.setRows(40);
	GrokFileGenerator gfg = new GrokFileGenerator(true);
	gfg.generateFileString();
	String grokFile = gfg.getOptimizedGrokFile();
	grokContent.setValue(grokFile);
	grokContent.setReadOnly(true);
	// Generate fields for upload handling, fill them with data
	Label uploadTo = new Label("Upload to:");
	rightLayout.addComponent(uploadTo);
	final OptionGroup selector = new OptionGroup();
	selector.addItem("Host");
	selector.addItem("Group");
	rightLayout.addComponent(selector);
	selector.setImmediate(true);
	final VerticalLayout uploadPanel = new VerticalLayout();
	rightLayout.addComponent(uploadPanel);
	selector.addListener(new Property.ValueChangeListener(){

	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.data.Property.ValueChangeListener#valueChange(com.vaadin.data.Property.ValueChangeEvent)
	     */
	    public void valueChange(ValueChangeEvent event) {
		uploadPanel.removeAllComponents();
		final Select select = new Select();
		select.setImmediate(true);
		if (selector.getValue().toString().equals("Host")){
		    Label partHost = new Label("Particular host:");
		    uploadPanel.addComponent(partHost);
		    HostDAO hDal = new HostDAO();
		    List<Host> allHosts = hDal.getAllHosts();
		    for (Host i: allHosts){
			select.addItem(i.getHost());
		    }
		    hDal.closeSession();
		}else{
		    Label partGroup = new Label("Particular group:");
		    uploadPanel.addComponent(partGroup);
		    GroupDAO gDal = new GroupDAO();
		    List<Group> allGroups = gDal.getAllGroups();
		    for (Group i: allGroups){
			select.addItem(i.getGroupid());
		    }
		    gDal.closeSession();
		}
		uploadPanel.addComponent(select);
		Button submitButton = new Button("Upload file");
		uploadPanel.addComponent(submitButton);
		submitButton.addClickListener(new Button.ClickListener() {
		    
		    @Override
		    /*
		     * (non-Javadoc)
		     * @see com.vaadin.ui.Button.ClickListener#buttonClick(com.vaadin.ui.Button.ClickEvent)
		     */
		    public void buttonClick(ClickEvent event) {
			try {
			    // Create temporary file
			    File f = new File("/tmp/grokblacklist");
			    f.createNewFile();
			    BufferedWriter tmpGrokFileWrt = new BufferedWriter(new OutputStreamWriter(new FileOutputStream("/tmp/grokblacklist")));		
			    String grokFile = grokContent.getValue();
			    String[] grokParsedFile = grokFile.split("\\n");
			    for (String i: grokParsedFile){
				tmpGrokFileWrt.append(i);
			    }
			    // Upload the file
			    if (selector.getValue().toString().equals("Host")){
				HostDAO hDal = new HostDAO();
				Host uploadHost = hDal.findHost(select.getValue().toString());
				if (uploadHost != null){
				    ConfigLoader config = new ConfigLoader();
				    config.loadConfig();
				    CommonFileUploadBroker fupb = new CommonFileUploadBroker("/tmp/grokblacklist", 
					uploadHost.getIp(), config.getGrokBlacklistFile());
				    fupb.uploadFile();
				    Notification.show("File upload was started, please check the destination host");
				}else Notification.show("Host not found","Please check that you've selected correct host",Notification.TYPE_ERROR_MESSAGE);
				hDal.closeSession();
			    }else{
				HostDAO hDal = new HostDAO();
				List<Host> hosts = hDal.getHostsByGroup(select.getValue().toString());
				if (hosts.size()>0){
				    for (Host i: hosts){
					ConfigLoader config = new ConfigLoader();
					config.loadConfig();
					CommonFileUploadBroker fupb = new CommonFileUploadBroker("/tmp/grokblacklist", 
						i.getIp(), config.getGrokRulesFile());
					fupb.uploadFile();
				    }
				    Notification.show("File uploads were started, please check the destination hosts");
				}else Notification.show("Upload failed","No host was found in given group",Notification.TYPE_ERROR_MESSAGE);
				hDal.closeSession();
			    }
			    // Delete the file at the end
			    f.delete();
			} catch (FileNotFoundException e) {
			    System.out.println("File was not found");
			    e.printStackTrace();
			} catch (IOException  e){
			    System.out.println("File could not be created");
			    e.printStackTrace();
			}
		    }
		});
	    }
	});
    }
    @Override
    /*
     * (non-Javadoc)
     * @see com.vaadin.navigator.View#enter(com.vaadin.navigator.ViewChangeListener.ViewChangeEvent)
     */
    public void enter(ViewChangeEvent event) {
	init();	
    }
}
