package com.example.groracle.Views;

import java.util.ArrayList;

import com.example.groracle.Menu;
import com.example.groracle.Brokers.CommonPermissionsBroker;
import com.example.groracle.Brokers.PasswordHandler;
import com.example.groracle.DAL.DAO.UserDAO;
import com.example.groracle.Entity.User;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener.ViewChangeEvent;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.Label;
import com.vaadin.ui.Notification;
import com.vaadin.ui.Panel;
import com.vaadin.ui.PasswordField;
import com.vaadin.ui.TextArea;
import com.vaadin.ui.TextField;
import com.vaadin.ui.VerticalLayout;
/**
 * Class representing UI implementation od adding new user to the system
 * @author tdosek
 * @version 1.0
 */
public class NewUserView extends Panel implements View {
    public static final String NAME = "NewUser";
    private Navigator navigator = null;
    private VerticalLayout header = new VerticalLayout();
    /**
     * Constructor of the class
     * @param navigator variable representing UI navigator class which handles UI element switching 
     */
    public NewUserView(Navigator navigator){
	this.navigator = navigator;
    }
    /**
     * Initialization method for UI layout 
     */
    private void init(){
	setContent(header);
	header.removeAllComponents();
	Menu menu = new Menu(navigator);
	menu.initMenu();
	header.addComponent(menu);
	initEditor();
    }
    /**
     * Initialization method for UI layout of the editor
     */
    private void initEditor(){
	VerticalLayout editor = new VerticalLayout();
	editor.setStyleName("loginEditor");
	header.addComponent(editor);
	
	Label usernameL = new Label("Username*");
	editor.addComponent(usernameL);
	final TextField userF = new TextField("");
	editor.addComponent(userF);
	usernameL.setStyleName("loginName");
	userF.setStyleName("loginUserText");
	
	Label descL = new Label("Description");
	editor.addComponent(descL);
	final TextArea descT = new TextArea("");
	descT.setWidth("400px");
	descT.setRows(20);
	editor.addComponent(descT);
	descL.setStyleName("loginName");
	descT.setStyleName("loginUserText");
	
	Label emailL = new Label("E-mail*");
	editor.addComponent(emailL);
	final TextField emailT = new TextField("");
	editor.addComponent(emailT);
	emailL.setStyleName("loginName");
	emailT.setStyleName("loginUserText");
	
	Label newLabel = new Label("New password*");
	editor.addComponent(newLabel);
	final PasswordField newPass = new PasswordField("");
	editor.addComponent(newPass);
	newLabel.setStyleName("loginName");
	newPass.setStyleName("loginUserText");
	
	Label newConfLabel = new Label("Confirm password*");
	editor.addComponent(newConfLabel);
	final PasswordField newConfPass = new PasswordField("");
	editor.addComponent(newConfPass);
	newConfLabel.setStyleName("loginName");
	newConfPass.setStyleName("loginUserText");
	
	Button newUserB = new Button("Create user");
	newUserB.setStyleName("loginButton");
	editor.addComponent(newUserB);
	newUserB.addClickListener(new ClickListener() {
	    
	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.ui.Button.ClickListener#buttonClick(com.vaadin.ui.Button.ClickEvent)
	     */
	    public void buttonClick(ClickEvent event) {
		PasswordHandler ph = new PasswordHandler();
		String user = userF.getValue();
		String description = descT.getValue();
		String email = emailT.getValue();
		String newPassword = newPass.getValue();
		String newConfirmedPass = newConfPass.getValue();
		if(newPassword.equals(newConfirmedPass)){
		    if (!emailT.getValue().equals(null)){
			User newUser = new User();
			newUser.setUsername(user);
			newUser.setDescription(description);
			newUser.setEmail(email);
			newUser.setRole("user");
			UserDAO userDal = new UserDAO();
			if(userDal.newUser(newUser)){
			    ph.setNewPassword(user, newPassword, emailT.getValue());
			    CommonPermissionsBroker pb = new CommonPermissionsBroker();
			    ArrayList<String> newPermissions = new ArrayList<>();
			    newPermissions.add("EditPattern");
			    pb.changePermissionToUser(user, newPermissions);
			    init();
			    Notification.show("User successfully created");
			}else Notification.show("Failed to create user","Username already exists",Notification.TYPE_ERROR_MESSAGE);
			userDal.closeSession();
		    }else Notification.show("Failed to create password","No e-mail was provided", Notification.TYPE_ERROR_MESSAGE);
		}else Notification.show("Failed to create password","New password and its confirmation do not match", Notification.TYPE_ERROR_MESSAGE);
	    }
	});
    }
    @Override
    /*
     * (non-Javadoc)
     * @see com.vaadin.navigator.View#enter(com.vaadin.navigator.ViewChangeListener.ViewChangeEvent)
     */
    public void enter(ViewChangeEvent event) {
	init();	
    }

}
